import { type DocumentReference, type DocumentSnapshot, type Firestore, type Query as FirestoreQuery, Transaction, type WriteBatch } from '@firebase/firestore';
import type { AggregateQuery, Aggregated } from 'firestore-repository/aggregate';
import type { WriteDocumentData } from 'firestore-repository/document';
import type { Query } from 'firestore-repository/query';
import type * as repository from 'firestore-repository/repository';
import { type CollectionSchema, type DocPathElement, type Id, type Model, type ParentId } from 'firestore-repository/schema';
export type Env = {
    transaction: Transaction;
    writeBatch: WriteBatch;
    query: FirestoreQuery;
};
export type TransactionOption = repository.TransactionOption<Env>;
export type WriteTransactionOption = repository.WriteTransactionOption<Env>;
export declare class Repository<T extends CollectionSchema = CollectionSchema> implements repository.Repository<T, Env> {
    readonly collection: T;
    readonly db: Firestore;
    constructor(collection: T, db: Firestore);
    get(id: Id<T>, options?: TransactionOption): Promise<Model<T> | undefined>;
    getOnSnapshot(id: Id<T>, next: (snapshot: Model<T> | undefined) => void, error?: (error: Error) => void, complete?: () => void): repository.Unsubscribe;
    list(query: Query<T>): Promise<Model<T>[]>;
    listOnSnapshot(query: Query<T>, next: (snapshot: Model<T>[]) => void, error?: (error: Error) => void, complete?: () => void): repository.Unsubscribe;
    aggregate<U extends AggregateQuery<T>>(aggregate: U): Promise<Aggregated<U>>;
    set(doc: Model<T>, options?: WriteTransactionOption): Promise<void>;
    delete(id: Id<T>, options?: WriteTransactionOption): Promise<void>;
    batchSet(docs: Model<T>[], options?: WriteTransactionOption): Promise<void>;
    batchDelete(ids: Id<T>[], options?: WriteTransactionOption): Promise<void>;
    protected batchWriteOperation<U>(targets: U[], runner: {
        batch: (batch: WriteBatch, target: U) => void;
        transaction: (transaction: Transaction, target: U) => void;
    }, options?: WriteTransactionOption): Promise<void>;
    protected docRef(id: Id<T>): DocumentReference<import("@firebase/firestore").DocumentData, import("@firebase/firestore").DocumentData>;
    protected collectionRef(parentId: ParentId<T>): import("@firebase/firestore").CollectionReference<import("@firebase/firestore").DocumentData, import("@firebase/firestore").DocumentData>;
    protected fromFirestore(doc: DocumentSnapshot): Model<T> | undefined;
    protected toFirestoreData(data: Model<T>): WriteDocumentData;
}
/**
 * Obtain document path elements from DocumentReference
 */
export declare const docPathElements: (doc: DocumentReference) => [DocPathElement, ...DocPathElement[]];
export declare const toFirestoreQuery: (db: Firestore, query: Query) => FirestoreQuery;
